from typing import Any

from ape.api import ConverterAPI, Web3Provider
from ape.exceptions import ProviderError
from ape.logging import logger
from ape.types import AddressType
from ape.utils import cached_property
from web3.main import ENS


class ENSConversions(ConverterAPI):
    """Converts ENS names like `my-name.eth` to `0xAbCd...1234`"""

    @cached_property
    def ens(self) -> ENS:
        provider = self.networks.active_provider

        if not provider:
            raise ProviderError("Not connected to a provider.")

        if not isinstance(provider, Web3Provider):
            raise NotImplementedError("Currently, only web3 providers work with this plugin.")

        web3 = provider._web3

        if not hasattr(web3, "ens"):
            raise NotImplementedError("This provider does not implement ENS calls.")

        return web3.ens

    def is_convertible(self, value: Any) -> bool:
        if not isinstance(value, str):
            return False

        elif "." not in value:
            return False

        elif not ENS.is_valid_name(value):
            return False

        elif not self.ens:
            logger.warning("Ethereum Mainnet provider not available for ENS address lookups")
            return False

        else:
            return self.ens.address(value) is not None

    def convert(self, value: str) -> AddressType:
        return self.ens.address(value)



import pytest
from ape.api import ProviderAPI, Web3Provider
from ape.api.config import ConfigItem
from ape.exceptions import ProviderError
from ape.managers.converters import ConversionManager
from ape.managers.networks import NetworkManager

from ape_ens.converters import ENSConversions


@pytest.fixture
def mock_networks(mocker):
    return mocker.MagicMock(spec=NetworkManager)


@pytest.fixture
def mock_config(mocker):
    return mocker.MagicMock(spec=ConfigItem)


@pytest.fixture
def mock_converter(mocker):
    return mocker.MagicMock(spec=ConversionManager)


@pytest.fixture
def converter(mocker, mock_networks, mock_config, mock_converter):
    mock_networks.active_provider = mocker.MagicMock(spec=Web3Provider)
    return ENSConversions(config=mock_config, networks=mock_networks, converter=mock_converter)


def test_ens_when_no_provider(mock_config, mock_networks, mock_converter):
    mock_networks.active_provider = None

    with pytest.raises(ProviderError) as err:
        _ = ENSConversions(config=mock_config, networks=mock_networks, converter=mock_converter).ens

    assert str(err.value) == "Not connected to a provider."


def test_ens_when_not_web3_provider(mocker, mock_config, mock_networks, mock_converter):
    mock_networks.active_provider = mocker.MagicMock(spec=ProviderAPI)

    with pytest.raises(NotImplementedError) as err:
        _ = ENSConversions(config=mock_config, networks=mock_networks, converter=mock_converter).ens

    assert str(err.value) == "Currently, only web3 providers work with this plugin."


def test_is_convertible(converter):
    assert converter.is_convertible("test.eth")


def test_is_not_convertible(converter):
    assert not converter.is_convertible("test")
    assert not converter.is_convertible("0xe1122aa5533228143C4Ce8fC4642aa33b857B332")
    assert not converter.is_convertible(23452345)



from ape import plugins
from ape.types import AddressType

from .converters import ENSConversions


@plugins.register(plugins.ConversionPlugin)
def converters():
    yield AddressType, ENSConversions
